Transform the database_dir/database_root arg into an absolute path.
[adu.git] / string.c
index aec62ae9918f298578ec4881d7e9eebd63a1197f..13555550efb6e1e567f66ca6f03158b79645ed7e 100644 (file)
--- a/string.c
+++ b/string.c
@@ -4,7 +4,7 @@
  * Licensed under the GPL v2. For licencing details see COPYING.
  */
 
-/** \file string.c Memory allocation and string handling functions. */
+/** \file string.c \brief Memory allocation and string handling functions. */
 
 #include "adu.h"
 #include "string.h"
@@ -12,7 +12,7 @@
 #include "error.h"
 
 /**
- * Paraslash's version of realloc().
+ * Adu's version of realloc().
  *
  * \param p Pointer to the memory block, may be \p NULL.
  * \param size The desired new size.
@@ -41,7 +41,7 @@ __must_check __malloc void *adu_realloc(void *p, size_t size)
 }
 
 /**
- * Paraslash's version of malloc().
+ * Adu's version of malloc().
  *
  * \param size The desired new size.
  *
@@ -66,7 +66,7 @@ __must_check __malloc void *adu_malloc(size_t size)
 }
 
 /**
- * Paraslash's version of calloc().
+ * Adu's version of calloc().
  *
  * \param size The desired new size.
  *
@@ -86,7 +86,7 @@ __must_check __malloc void *adu_calloc(size_t size)
 }
 
 /**
- * Paraslash's version of strdup().
+ * Adu's version of strdup().
  *
  * \param s The string to be duplicated.
  *
@@ -138,7 +138,7 @@ __must_check __printf_1_2 __malloc char *make_message(const char *fmt, ...)
  * Append \p b to \p a.
  *
  * \return If \a a is \p NULL, return a pointer to a copy of \a b, i.e.
* para_strcat(NULL, b) is equivalent to para_strdup(b). If \a b is \p NULL,
  * return \a a without making a copy of \a a.  Otherwise, construct the
  * concatenation \a c, free \a a (but not \a b) and return \a c.
  *
@@ -196,14 +196,14 @@ __must_check int atoi64(const char *str, int64_t *result)
 }
 
 /**
- * Split string and return pointers to its parts.
+ * Split string and return pointers to its parts.
  *
  * \param args The string to be split.
  * \param argv_ptr Pointer to the list of substrings.
  * \param delim Delimiter.
  *
  * This function modifies \a args by replacing each occurance of \a delim by
- * zero. A \p NULL-terminated array of pointers to char* is allocated dynamically
+ * zero. A \p NULL terminated array of pointers to char* is allocated dynamically
  * and these pointers are initialized to point to the broken-up substrings
  * within \a args. A pointer to this array is returned via \a argv_ptr.
  *
@@ -245,10 +245,11 @@ __must_check unsigned split_args(char *args, char *** const argv_ptr, const char
        return n;
 }
 
-enum line_state_flags {LSF_HAVE_WORD = 1, LSF_BACKSLASH = 2, LSF_QUOTE = 4};
 
 static int get_next_word(const char *line, char **word)
 {
+       enum line_state_flags {LSF_HAVE_WORD = 1, LSF_BACKSLASH = 2,
+               LSF_QUOTE = 4};
        const char *in;
        char *out;
        int ret, state = 0;
@@ -320,6 +321,11 @@ out:
        return ret;
 }
 
+/**
+ * Free an array of words created by create_argv().
+ *
+ * \param argv A pointer previously obtained by \ref create_argv().
+ */
 void free_argv(char **argv)
 {
        int i;
@@ -330,6 +336,16 @@ void free_argv(char **argv)
 }
 
 /**
+ * Split a line into words which are separated by whitespace.
+ *
+ * In contrast to gengetopt's string parser, double quotes, backslash-escaped
+ * characters and special characters like \p \\n are honored. The result
+ * contains pointers to copies of the words contained in \a line and has to be
+ * freed by using \ref free_argv().
+ *
+ * \param line The line to be split.
+ * \param result The array of words is returned here.
+ *
  * \return Number of words in \a line, negative on errors.
  */
 int create_argv(const char *line, char ***result)
@@ -357,3 +373,35 @@ err:
        free(argv);
        return ret;
 }
+
+char *absolute_path(const char *path)
+{
+       char *cwd, *ap;
+       long int path_max;
+
+       if (!path || !path[0])
+               return NULL;
+       if (path[0] == '/')
+               return adu_strdup(path);
+
+#ifdef PATH_MAX
+       path_max = PATH_MAX;
+#else
+       /*
+        * The result of pathconf(3) may be huge and unsuitable for mallocing
+        * memory. OTOH pathconf(3) may return -1 to signify that PATH_MAX is
+        * not bounded.
+        */
+       path_max = pathconf(name, _PC_PATH_MAX);
+       if (path_max <= 0 || path_max >= 4096)
+               path_max = 4096;
+#endif
+       cwd = adu_malloc(path_max);
+       if (!getcwd(cwd, path_max)) {
+               free(cwd);
+               return NULL;
+       }
+       ap = make_message("%s/%s", cwd, path);
+       free(cwd);
+       return ap;
+}
@@ -138,7 +138,7 @@ __must_check __printf_1_2 __malloc char *make_message(const char *fmt, ...)
  *
  * \return If \a a is \p NULL, return a pointer to a copy of \a b, i.e.
- * para_strcat(NULL, b) is equivalent to para_strdup(b). If \a b is \p NULL,
+ * adu_strcat(NULL, b) is equivalent to adu_strdup(b). If \a b is \p NULL,
  * return \a a without making a copy of \a a.  Otherwise, construct the
  * concatenation \a c, free \a a (but not \a b) and return \a c.